-- % doesn't handle end of spline correctly - splines tend to have problems with end conditions

gr, li, ky, tm	= love.graphics, love.image, love.keyboard, love.timer
pi, sin, cos, atan2, floor = math.pi, math.sin, math.cos, math.atan2, math.floor

function love.load()
	path = {
		control		= {},
		steps		= 20,	-- number of control points
		resolution	= 40,	-- points between control points
		selected	= 1,	-- selected control point
		current		= 1,
		percent		= 0.25	-- actually fractional way along curve
		}
	width, height	= gr.getWidth(), gr.getHeight()
	calculate_points()
	love.graphics.setBackgroundColor( 55, 0, 255, 255 )
	gr.setCaption( 'CatMullRom Spline - Left click & drag to move highlighted point - a/s to change %' )
	point_selection()
	end

function point_selection()	-- for selection of % along curve
	total_points	= path.steps * path.resolution
	fraction		= math.floor( path.percent * total_points )	-- how far along curve
	sel_point		= math.floor( fraction / path.resolution )	-- closest point below
	if sel_point	< 1 then sel_point = 1 end
	if sel_point>#points - 2 then sel_points = #points - 2 end
	test_point		= path.control[sel_point]
	extra_steps		= fraction - sel_point*path.resolution		-- how many steps beyond point
	if extra_steps < 1 then extra_steps = 1 end
	end

function calculate_points()
	points		= {}
	local step	= 2 * pi / path.steps
	for i = 1, path.steps - 1 do
		path.control[i]		= {}
		path.control[i][1]	= width * i / path.steps
		path.control[i][2]	= ( height/2 - 20 ) * sin( step * ( i - 1 ) ) + height/2
		end
	end

function love.draw()
	love.graphics.setColor( 255, 255, 0, 255 )
	for i = 1, #path.control do
		love.graphics.circle( 'line', path.control[i][1], path.control[i][2], 10, 10 )
		end

	gr.setColor( 255, 0, 0, 255 )
	love.graphics.circle( 'line', path.control[path.selected][1], path.control[path.selected][2], 10, 10 )
	loc = percent_along_spline( path.control, path.resolution, sel_point, extra_steps )
	gr.setColor( 0, 255, 0, 255 )
	gr.circle( 'fill', loc[1], loc[2], 10 )
	gr.setColor( 255, 255, 255, 255 )
	gr.print('path percert:'..math.floor( path.percent * 100 ), 10, 20 )
	fps = love.timer.getFPS( )
	gr.print( 'fps: '..fps, 10, height-15 )
	end

function mouse_select()
	local mx, my	= love.mouse.getPosition( )
	local min		= width
	local ck		= #path.control
	for i = 1, #path.control do
		local dis = math.sqrt( ( path.control[i][1] - mx )^2+( path.control[i][2] - my )^2 )
		if dis < min then
			min	= dis
			ck	= i
			end
		end
	path.selected	= ck
	end

function love.update( dt )
	if love.mouse.isDown( 'l' ) then
		path.control[path.selected][1], path.control[path.selected][2] = love.mouse.getPosition( )
		end
	mouse_select()
	if ky.isDown('s') then path.percent=path.percent<1-dt/2 and path.percent+dt/2 or .999 point_selection() end
	if ky.isDown('a') then path.percent=path.percent>dt and path.percent-dt/2 or .01  point_selection() end
	end

function love.keypressed( key )
	if key	== 'r'		then calculate_points() end
	if key	== 'escape'	then love.event.push( 'quit' ) end
	end

function percent_along_spline( points, steps, pt, ex )	-- point file format {#,#}
	-- original points, # steps between points, point of interest, # steps beyond point
	if #points < 3 then	return points	end
	local steps = steps or 5
	local spline = {}
	local p0, p1, p2, p3, x, y
	local i = pt
		if i == 1 then					-- if first point
			p0, p1, p2, p3 = points[i], points[i], points[i + 1], points[i + 2]
			elseif	i >= #points-1 then		-- if last point
				p0, p1, p2, p3 = points[#points - 2], points[#points - 1], points[#points], points[#points]
			else
			p0, p1, p2, p3 = points[i - 1], points[i], points[i + 1], points[i + 2]
			end
		for t = 0, 1, 1 / steps do
			x = 0.5 * ( ( 2 * p1[1] ) + ( p2[1] - p0[1] ) * t + ( 2 * p0[1] - 5 * p1[1] + 4 * p2[1] - p3[1] ) * t * t + ( 3 * p1[1] - p0[1] - 3 * p2[1] + p3[1] ) * t * t * t )
			y = 0.5 * ( ( 2 * p1[2] ) + ( p2[2] - p0[2] ) * t + ( 2 * p0[2] - 5 * p1[2] + 4 * p2[2] - p3[2] ) * t * t + ( 3 * p1[2] - p0[2] - 3 * p2[2] + p3[2] ) * t * t * t )
			--prevent duplicate entries
			if not(#spline > 0 and spline[#spline][1] == x and spline[#spline][2] == y) then
				table.insert( spline , { x , y } )	-- table of points
				end
			end
	return spline[ex]
	end
